package com.fdage.dao.base;

import com.fdage.pojo.TbExhibitionCollection;

public interface TbExhibitionCollectionMapper {
    int deleteByPrimaryKey(Long id);

    int insert(TbExhibitionCollection record);

    int insertSelective(TbExhibitionCollection record);

    TbExhibitionCollection selectByPrimaryKey(Long id);

    int updateByPrimaryKeySelective(TbExhibitionCollection record);

    int updateByPrimaryKey(TbExhibitionCollection record);
}